/*
** Module   : ServerImpl.java
** Abstract : n class that implements the app server specific method and  attribute APIs.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CS  20130211 Created initial version.
** 002 CA  20130221 Added resourceDelete stub. setType was renamed to setResourceType. 
** 003 OM  20130304 Refactored isValid and isUnknown of WrappedResource to valid and unknown.
** 004 CS  20130321 Added missing versions of connect() method.
*/

package com.goldencode.p2j.util;

/**
 * A class for implementing the Server methods and attributes.
 */
public class ServerImpl
extends SensitiveResource 
implements Server,
           AsyncRequestCountAttribute,
           FirstLastProcedureAttribute,
           SslServerAttribute,
           SubTypeAttribute,
           ConnectableServer
{
   /**
    * Default constructor which initializes Server specific settings.
    */
   public ServerImpl()
   {
      // TODO implement more constructor initializations logic EX:
      setResourceType("SERVER");
      setUnknown(false);
   }
   
   /**
    * Constructor with PREV-SIBLING and NEXT-SIBLING as attributes.
    * 
    * @param   prevSibling
    *          The previously created Server handle
    * @param   nextSibling
    *          The following Server handle
    */
   public ServerImpl(handle prevSibling, handle nextSibling)
   {
      this();
      setPrevSibling(prevSibling);
      setNextSibling(nextSibling);
   }

   /**
    * Reports if this object is valid for use.
    *
    * @return   <code>true</code> if we are valid (can be used).
    */
   public boolean valid()
   {
      // TODO Implement the actual logic.
      return true;
   }

   /**
    * Returns the number of asynchronous requests.
    * 
    * @return   See above.
    */
   public integer getAsyncRequestCount()
   {
      // TODO Implement the actual logic.
      return new integer();
   }

   /**
    * Returns the subtype of the handle. 
    * <p>
    * For SERVER handles this represents the type of server to which this is connected. 
    * The possible values are "APPSERVER" and "WEBSERVICE".
    * 
    * @return   See above.
    */
   public character getSubType()
   {
      // TODO Implement the actual logic.
      return new character();
   }

   /**
    * Java implementation for getting the  SSL-SERVER-NAME  attribute  which contains the SSL 
    * server name if the connection is made as SSL.
    * 
    * @return   The SSL server name, if the connection is made as SSL or unknown
    *           otherwise
    */
   public character getSslServerName()
   {
      // TODO Implement the actual logic.
      return new character();
   }

   /**
    * Returns the first entry in the list of persistent procedures or procedure objects.
    * 
    * @return   See above.
    */
   public handle firstProcedure()
   {
      // TODO Implement the actual logic.
      return null;
   }

   /**
    * Returns the first entry in the list of persistent procedures or procedure objects.
    * 
    * @return   See above.
    */
   public handle lastProcedure()
   {
      // TODO Implement the actual logic.
      return null;
   }

   /**
    * This returns the id for the AppServer connection to this handle. For Web Services the return
    * value will be an empty string.
    * 
    * @return   See above.
    */
   public character getClientConnectionId()
   {
      // TODO Implement the actual logic.
      return new character();
   }

   /**
    * Returns a handle to the first asynchronous requests in the AppServer or WebService list of
    * asynchronous requests.
    * 
    * @return   See above.
    */
   public handle getFirstAsyncRequest()
   {
      // TODO Implement the actual logic.
      return null;
   }

   /**
    * Returns a handle to the last asynchronous requests in the AppServer or WebService list of
    * asynchronous requests.
    * 
    * @return   See above.
    */
   public handle getLastAsyncRequest()
   {
      // TODO Implement the actual logic.
      return null;
   }
   
   /**
    * This sends a stop condition to the AppServer or Web Service.
    * <p>
    * Depending on the type of server the stop condition is sent to, it may be handled  differently.
    * 
    * @return   <code>true</code> if CONNECTED() method returns <code>true</code>, 
    *           <code>false</code> otherwise.
    */
   public logical cancelRequests()
   {
      // TODO Implement the actual logic.
      return connected();
   }

   /**
    * Calls the {@link #cancelRequests()} method when the specified number of seconds have passed.
    * 
    * @param    timeSec 
    *           The time in seconds after the {@link #cancelRequests()} method will be called. 4GL
    *           allows only integer or int64 values for this parameter. 
    *         
    * @return   <code>false</code> if timer cannot be set or {@link #cancelRequests()}'s result 
    *           otherwise
    */
   public logical cancelRequestsAfter(integer timeSec)
   {
      // TODO Implement the actual logic.
      return cancelRequests();
   }
   
   /**
    * Calls the {@link #cancelRequests()} method when the specified number of seconds have passed.
    * 
    * @param    timeSec 
    *           The time in seconds after the {@link #cancelRequests()} method will be called. 4GL
    *           allows only integer or int64 values for this parameter. 
    *         
    * @return   <code>false</code> if timer cannot be set or {@link #cancelRequests()}'s result 
    *           otherwise
    */
   public logical cancelRequestsAfter(int64 timeSec)
   {
      // TODO Implement the actual logic.
      return cancelRequests();
   }
   
   /**
    * Calls the {@link #cancelRequests()} method when the specified number of seconds have passed.
    * 
    * @param    timeSec 
    *           The time in seconds after the {@link #cancelRequests()} method will be called. 4GL
    *           allows only integer or int64 values for this parameter. 
    *         
    * @return   <code>false</code> if timer cannot be set or {@link #cancelRequests()}'s result 
    *           otherwise
    */
   public logical cancelRequestsAfter(long timeSec)
   {
      return cancelRequestsAfter(new int64(timeSec));
   }

   /**
    * Method declaration  for the CONNECT() procedure supported by sockets web-services and server
    * handles.
    *  
    * @param    options
    *           The set of parameters to establish the connect procedure. 4GL allows only
    *           character values for this parameter.
    * 
    * @return   <code>true</code> if connection succeeded, or <code>false</code>
    *           otherwise
    */
   public logical connect(character options)
   {
      return connect(options, new character(), new character(), new character());
   }

   /**
    * Method declaration  for the CONNECT() procedure supported by sockets
    * web-services and server handles.
    *  
    * @param    options
    *           The set of parameters to establish the connect procedure. 4GL allows only 
    *           {@link character} values for this parameter.
    * 
    * @return   <code>true</code> if connection succeeded, or <code>false</code>
    *           otherwise
    */
   public logical connect(String options)
   {
      return connect(new character(options));
   }
   
   /**
    * Method declaration  for the CONNECT() procedure supported by sockets and
    * web-services.
    *  
    * @param    options
    *           The set of parameters to establish the connect procedure. 4GL allows only  
    *           character values for this parameter.
    * @param    user
    *           The user with which the connection is made.
    * @param    pwd
    *           The password for the user with which the connection is made
    * @param    serverInfo
    *           Server info
    * 
    * @return   <code>true</code> if connection succeeded, or <code>false</code>
    *           otherwise
    */
   public logical connect(character options, character user, character pwd, character serverInfo)
   {
      // TODO Implement the actual logic.
      return new logical(true);
   }
   
   /**
    * Method declaration  for the CONNECT() procedure supported by sockets and
    * web-services.
    *  
    * @param    options
    *           The set of parameters to establish the connect procedure. 4GL allows only  
    *           character values for this parameter.
    * @param    user
    *           The user with which the connection is made.
    * @param    pwd
    *           The password for the user with which the connection is made
    * 
    * @return   <code>true</code> if connection succeeded, or <code>false</code>
    *           otherwise
    */
   public logical connect(character options, character user, character pwd)
   {
      return connect(options, user, pwd, new character());
   }
   
   /**
    * Method declaration  for the CONNECT() procedure supported by sockets and
    * web-services.
    *  
    * @param    options
    *           The set of parameters to establish the connect procedure. 4GL allows only  
    *           character values for this parameter.
    * @param    user
    *           The user with which the connection is made.
    * 
    * @return   <code>true</code> if connection succeeded, or <code>false</code>
    *           otherwise
    */
   public logical connect(character options, character user)
   {
      return connect(options, user, new character(), new character());
   }

   /**
    * Method declaration  for the CONNECTED() procedure supported by sockets web-services and 
    * server handles, this will return <code>true</code> if the socket is connected or 
    * <code>false</code> otherwise.
    * 
    * @return   <code>true</code> if the socket is connected or <code>false</code> otherwise.
    */
   public logical connected()
   {
      // TODO Implement the actual logic.
      return new logical(true);
   }

   /**
    * Method declaration  for the DISCONNECT() procedure supported by sockets web-services and 
    * server handles, this will attempt to disconnect the resource and returns <code>true</code>
    * if succeeded or <code>false</code> otherwise.
    * 
    * @return   <code>true</code> if disconnect succeeded or <code>false</code> otherwise.
    */
   public logical disconnect()
   {
      // TODO Implement the actual logic.
      return new logical(true);
   }
   
   /**
    * Method that specifies the remote procedures the client can execute from the AppServer 
    * session.
    * 
    * @param    attrList
    *           The list of procedure  and  name-patterns separated by comma to be exported.
    *         
    * @return   <true> if REMOTE attribute is true and operation succeeded or
    *           <false> otherwise
    */
   public static logical export(String attrList)
   {
      // TODO implement actual logic
      return new logical(true);
   }
   
   /**
    * Method that specifies the remote procedures the client can execute from the AppServer 
    * session.
    * 
    * @param    attrList
    *           The list of procedure  and  name-patterns separated by comma to be exported.
    *         
    * @return   <code>true</code> if REMOTE attribute is true and operation succeeded or
    *           <code>false</code> otherwise
    */
   public static logical export(character attrList)
   {
      // TODO implement actual logic
      return new logical(true);
   }
   
   /**
    * Method implementation for getting the SERVER-OPERATING-MODE attribute of the session handle.
    * <p>
    * Possible values:
    * <ul>
    *    <li>State-reset
    *    <li>State-aware
    *    <li>Stateless
    *    <li>State-free
    * </ul>
    * 
    * @return   The corresponding SERVER-OPERATING-MODE attribute of the session handle.
    */
   public static character getServerOperatingMode()
   {
      // TODO implement actual logic
      return new character();
   }
   
   /**
    * Indicates if the session is an AppServer session or not
    * 
    * @return   <code>true</code> if the current session is an AppServer session or false 
    * otherwise.
    */
   public static logical isRemote()
   {
      // TODO implement actual logic
      return new logical(true);
   }
   
   /**
    * Delete the resource.
    */
   @Override
   protected void resourceDelete()
   {
      // TODO Auto-generated method stub
   }
}
